@@ -28,14 +28,15 @@ import subprocess
 import time
 import yaml
 import requests
-import medusa.utils
+from medusa.utils import evaluate_boolean, null_if_empty
 
 from subprocess import PIPE
 from retrying import retry
+from cassandra import ProtocolVersion
 from cassandra.cluster import Cluster, ExecutionProfile
 from cassandra.policies import WhiteListRoundRobinPolicy
 from cassandra.auth import PlainTextAuthProvider
-from ssl import SSLContext, PROTOCOL_TLSv1, CERT_REQUIRED
+from ssl import SSLContext, PROTOCOL_TLS, CERT_REQUIRED
 from medusa.network.hostname_resolver import HostnameResolver
 
 
@@ -59,19 +60,19 @@ class CqlSessionProvider(object):
         self._ssl_context = None
         self._cassandra_config = cassandra_config
 
-        if cassandra_config.cql_username is not None and cassandra_config.cql_password is not None:
+        if null_if_empty(cassandra_config.cql_username) and null_if_empty(cassandra_config.cql_password):
             auth_provider = PlainTextAuthProvider(username=cassandra_config.cql_username,
                                                   password=cassandra_config.cql_password)
             self._auth_provider = auth_provider
 
-        if cassandra_config.certfile is not None and cassandra_config.usercert is not None and \
-           cassandra_config.userkey is not None:
-            ssl_context = SSLContext(PROTOCOL_TLSv1)
+        if cassandra_config.certfile is not None:
+            ssl_context = SSLContext(PROTOCOL_TLS)
             ssl_context.load_verify_locations(cassandra_config.certfile)
             ssl_context.verify_mode = CERT_REQUIRED
-            ssl_context.load_cert_chain(
-                certfile=cassandra_config.usercert,
-                keyfile=cassandra_config.userkey)
+            if cassandra_config.usercert is not None and cassandra_config.userkey is not None:
+                ssl_context.load_cert_chain(
+                    certfile=cassandra_config.usercert,
+                    keyfile=cassandra_config.userkey)
             self._ssl_context = ssl_context
 
         load_balancing_policy = WhiteListRoundRobinPolicy(ip_addresses)
@@ -89,7 +90,8 @@ class CqlSessionProvider(object):
         cluster = Cluster(contact_points=self._ip_addresses,
                           auth_provider=self._auth_provider,
                           execution_profiles=self._execution_profiles,
-                          ssl_context=self._ssl_context)
+                          ssl_context=self._ssl_context,
+                          protocol_version=ProtocolVersion.V4)
 
         if retry:
             max_retries = 5
@@ -422,7 +424,7 @@ class Cassandra(object):
             # Eventually I think we will want to introduce an abstraction layer for Cassandra's
             # API that Medusa requires. There should be an implementation for using nodetool,
             # one for Jolokia, and a 3rd for the management sidecard used by Cass Operator.
-            if medusa.utils.evaluate_boolean(self.kubernetes_config.enabled):
+            if evaluate_boolean(self.kubernetes_config.enabled):
                 data = {
                     "type": "exec",
                     "mbean": "org.apache.cassandra.db:type=StorageService",
@@ -445,7 +447,7 @@ class Cassandra(object):
         cmd = self.delete_snapshot_command(tag)
         if self.snapshot_exists(tag):
 
-            if medusa.utils.evaluate_boolean(self.kubernetes_config.enabled):
+            if evaluate_boolean(self.kubernetes_config.enabled):
                 data = {
                     "type": "exec",
                     "mbean": "org.apache.cassandra.db:type=StorageService",
