@@ -28,8 +28,9 @@ from behave import given, when, then
 from pathlib import Path
 from subprocess import PIPE
 import signal
+from cassandra import ProtocolVersion
 from cassandra.cluster import Cluster
-from ssl import SSLContext, PROTOCOL_TLSv1, CERT_REQUIRED
+from ssl import SSLContext, PROTOCOL_TLS, PROTOCOL_TLSv1, PROTOCOL_TLSv1_1, PROTOCOL_TLSv1_2, CERT_REQUIRED
 
 import medusa.backup_node
 import medusa.index
@@ -148,8 +149,6 @@ def _i_have_a_fresh_ccm_cluster_running(context, cluster_name, client_encryption
         ]
     )
 
-    os.popen("ccm node1 updateconf 'storage_port: 7011'").read()
-
     if client_encryption == 'with_client_encryption':
         is_client_encryption_enable = True
         update_client_encrytion_opts = "ccm node1 updateconf -y 'client_encryption_options: { enabled: true,\
@@ -200,8 +199,6 @@ def _i_have_a_fresh_ccm_cluster_with_jolokia_running(context, cluster_name, clie
         ]
     )
 
-    os.popen("ccm node1 updateconf 'storage_port: 7011'").read()
-
     if client_encryption == 'with_client_encryption':
         is_client_encryption_enable = True
         update_client_encrytion_opts = "ccm node1 updateconf -y 'client_encryption_options: { enabled: true,\
@@ -312,6 +309,7 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
             "concurrent_transfers": 4,
             "prefix": storage_prefix,
             "aws_cli_path": "aws",
+            "host": "s3.eu.cloud-object-storage.appdomain.cloud",
             "region": "eu-smart",
             "transfer_max_bandwidth": "1MB/s"
         }
@@ -331,7 +329,8 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
             os.path.join(
                 "~/.ccm",
                 "repository",
-                context.cassandra_version,
+                context.cassandra_version.replace(
+                    "github:", "githubCOLON").replace("/", "SLASH"),
                 "bin",
                 "sstableloader",
             )
@@ -556,6 +555,16 @@ def _i_perform_grpc_backup_of_node_named_backupname(context, backup_mode, backup
     context.grpc_client.backup(backup_name, backup_mode)
 
 
+@when(r'I perform a backup over gRPC in "{backup_mode}" mode of the node named "{backup_name}" and it fails')
+def _i_perform_grpc_backup_of_node_named_backupname_fails(context, backup_mode, backup_name):
+    try:
+        context.grpc_client.backup(backup_name, backup_mode)
+        raise AssertionError("Backup process should have failed but didn't.")
+    except Exception:
+        # This exception is required to be raised to validate the step
+        pass
+
+
 @then(r'I verify over gRPC that the backup "{backup_name}" exists')
 def _i_verify_over_grpc_backup_exists(context, backup_name):
     found = False
@@ -572,6 +581,16 @@ def _i_delete_backup_grpc(context, backup_name):
     context.grpc_client.delete_backup(backup_name)
 
 
+@then(r'I delete the backup "{backup_name}" over gRPC and it fails')
+def _i_delete_backup_grpc_fail(context, backup_name):
+    try:
+        context.grpc_client.delete_backup(backup_name)
+        raise AssertionError("Backup deletion should have failed but didn't.")
+    except Exception:
+        # This exception is required to be raised to validate the step
+        pass
+
+
 @then(r'I verify over gRPC the backup "{backup_name}" does not exist')
 def _i_verify_over_grpc_backup_does_not_exist(context, backup_name):
     assert not context.grpc_client.backup_exists(backup_name)
@@ -621,6 +640,36 @@ def _i_cannot_see_the_backup_named_backupname_when_i_list_the_backups(
     assert found is False
 
 
+@then('I cannot see purged backup files for the "{table_name}" table in keyspace "{keyspace}"')
+def _i_cannot_see_purged_backup_files_for_the_tablename_table_in_keyspace_keyspacename(
+    context, table_name, keyspace
+):
+    storage = Storage(config=context.medusa_config.storage)
+    path = os.path.join(
+        storage.prefix_path + context.medusa_config.storage.fqdn, "data", keyspace, table_name
+    )
+    sb_files = len(storage.storage_driver.list_objects(path))
+
+    node_backups = storage.list_node_backups()
+    # Parse its manifest
+    nb_list = list(node_backups)
+    nb_files = {}
+    for nb in nb_list:
+        manifest = json.loads(nb.manifest)
+        for section in manifest:
+            if (
+                section["keyspace"] == keyspace
+                and section["columnfamily"][: len(table_name)] == table_name
+            ):
+                for objects in section["objects"]:
+                    nb_files[objects["path"]] = 0
+
+    if sb_files != len(nb_files):
+        logging.error("{} objects found on remote storage and {} objects found on backups manifest".format(
+            sb_files, len(nb_files)))
+        assert sb_files == len(nb_files)
+
+
 @then('I can see the backup status for "{backup_name}" when I run the status command')
 def _i_can_see_backup_status_when_i_run_the_status_command(context, backup_name):
     medusa.status.status(config=context.medusa_config, backup_name=backup_name)
@@ -715,6 +764,13 @@ def _i_can_download_the_backup_single_table_successfully(context, backup_name, f
     cleanup(download_path)
 
 
+@then(r'Test TLS version connections if "{client_encryption}" is turned on')
+def _i_can_connect_using_all_tls_versions(context, client_encryption):
+    if client_encryption == 'with_client_encryption':
+        for tls_version in [PROTOCOL_TLSv1, PROTOCOL_TLSv1_1, PROTOCOL_TLSv1_2]:
+            connect_cassandra(True, tls_version)
+
+
 @when(r'I restore the backup named "{backup_name}"')
 def _i_restore_the_backup_named(context, backup_name):
     medusa.restore_node.restore_node(
@@ -1073,7 +1129,7 @@ def _i_delete_the_backup_named(context, backup_name, all_nodes=False):
                                backup_name=backup_name, all_nodes=all_nodes)
 
 
-def connect_cassandra(is_client_encryption_enable):
+def connect_cassandra(is_client_encryption_enable, tls_version=PROTOCOL_TLS):
     connected = False
     attempt = 0
     session = None
@@ -1081,7 +1137,7 @@ def connect_cassandra(is_client_encryption_enable):
 
     if is_client_encryption_enable:
 
-        ssl_context = SSLContext(PROTOCOL_TLSv1)
+        ssl_context = SSLContext(tls_version)
         ssl_context.load_verify_locations(certfile)
         ssl_context.verify_mode = CERT_REQUIRED
         ssl_context.load_cert_chain(
@@ -1091,13 +1147,18 @@ def connect_cassandra(is_client_encryption_enable):
 
     while not connected and attempt < 10:
         try:
-            cluster = Cluster(contact_points=["127.0.0.1"], ssl_context=_ssl_context)
+            cluster = Cluster(contact_points=["127.0.0.1"],
+                              ssl_context=_ssl_context,
+                              protocol_version=ProtocolVersion.V4)
             session = cluster.connect()
             connected = True
         except cassandra.cluster.NoHostAvailable:
             attempt += 1
             time.sleep(10)
 
+    if tls_version is not PROTOCOL_TLS:  # other TLS versions used for testing, close the session
+        session.shutdown()
+
     return session
 
 
